@@ -1,6 +1,4 @@
 //! provides diffing algorithm which returns patches
-#![allow(clippy::type_complexity)]
-#![allow(unused)]
 use crate::{
     node::attribute::group_attributes_per_name, Attribute, Element, Node,
     Patch, TreePath,
@@ -64,7 +62,7 @@ where
     diff_recursive(
         old_node,
         new_node,
-        &[],
+        &TreePath::root(),
         key,
         &|_old, _new| false,
         &|_old, _new| false,
@@ -105,26 +103,7 @@ where
         &'a Node<NS, TAG, LEAF, ATT, VAL>,
     ) -> bool,
 {
-    diff_recursive(old_node, new_node, &[], key, skip, rep)
-}
-
-/// returns true if all of the node children has key in their attributes
-#[allow(unused)]
-fn is_all_children_keyed<NS, TAG, LEAF, ATT, VAL>(
-    element: &Element<NS, TAG, LEAF, ATT, VAL>,
-    key: &ATT,
-) -> bool
-where
-    NS: PartialEq + Clone + Debug,
-    TAG: PartialEq + Debug,
-    LEAF: PartialEq + Clone + Debug,
-    ATT: PartialEq + Clone + Debug,
-    VAL: PartialEq + Clone + Debug,
-{
-    element
-        .get_children()
-        .iter()
-        .all(|child| is_keyed_node(child, key))
+    diff_recursive(old_node, new_node, &TreePath::root(), key, skip, rep)
 }
 
 /// returns true if any of the children of this element has key in their attributes
@@ -140,7 +119,7 @@ where
     VAL: PartialEq + Clone + Debug,
 {
     element
-        .get_children()
+        .children
         .iter()
         .any(|child| is_keyed_node(child, key))
 }
@@ -215,7 +194,7 @@ where
 fn diff_recursive<'a, 'b, NS, TAG, LEAF, ATT, VAL, SKIP, REP>(
     old_node: &'a Node<NS, TAG, LEAF, ATT, VAL>,
     new_node: &'a Node<NS, TAG, LEAF, ATT, VAL>,
-    path: &[usize],
+    path: &TreePath,
     key: &ATT,
     skip: &SKIP,
     rep: &REP,
@@ -240,37 +219,34 @@ where
         return vec![];
     }
 
-    let mut patches = vec![];
-
+    // replace node and return early
     if should_replace(old_node, new_node, key, rep) {
-        patches.push(Patch::replace_node(
+        return vec![Patch::replace_node(
             old_node.tag(),
-            TreePath::new(path.to_vec()),
+            path.clone(),
             new_node,
-        ));
-        return patches;
+        )];
     }
 
+    let mut patches = vec![];
+
     // The following comparison can only contain identical variants, other
     // cases have already been handled above by comparing variant
     // discriminants.
     match (old_node, new_node) {
         (Node::Leaf(old_leaf), Node::Leaf(new_leaf)) => {
             if old_leaf != new_leaf {
-                let ct = Patch::replace_node(
-                    old_node.tag(),
-                    TreePath::new(path.to_vec()),
-                    new_node,
-                );
+                let ct =
+                    Patch::replace_node(old_node.tag(), path.clone(), new_node);
                 patches.push(ct);
             }
         }
         // We're comparing two element nodes
         (Node::Element(old_element), Node::Element(new_element)) => {
-            let test = is_any_children_keyed(old_element, key)
+            let diff_as_keyed = is_any_children_keyed(old_element, key)
                 || is_any_children_keyed(new_element, key);
 
-            if test {
+            if diff_as_keyed {
                 let keyed_patches = keyed::diff_keyed_elements(
                     old_element,
                     new_element,
@@ -316,7 +292,7 @@ fn diff_non_keyed_elements<'a, 'b, NS, TAG, LEAF, ATT, VAL, SKIP, REP>(
     old_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
     new_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
     key: &ATT,
-    path: &[usize],
+    path: &TreePath,
     skip: &SKIP,
     rep: &REP,
 ) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
@@ -346,15 +322,14 @@ where
     let min_count = cmp::min(old_child_count, new_child_count);
     for index in 0..min_count {
         // if we iterate trough the old elements, a new child_path is created for that iteration
-        let mut child_path = path.to_vec();
-        child_path.push(index);
+        let child_path = path.traverse(index);
 
         let old_child = &old_element
             .children
             .get(index)
             .expect("No old_node child node");
         let new_child =
-            &new_element.children.get(index).expect("No new chold node");
+            &new_element.children.get(index).expect("No new child node");
 
         let more_patches =
             diff_recursive(old_child, new_child, &child_path, key, skip, rep);
@@ -364,75 +339,33 @@ where
     // If there are more new child than old_node child, we make a patch to append the excess element
     // starting from old_child_count to the last item of the new_elements
     if new_child_count > old_child_count {
-        let append_children_patch =
-            create_append_children_patch(old_element, new_element, path);
-        patches.push(append_children_patch);
+        patches.push(Patch::append_children(
+            &old_element.tag,
+            path.clone(),
+            new_element.children.iter().skip(old_child_count).collect(),
+        ));
     }
 
     if new_child_count < old_child_count {
-        let remove_node_patches =
-            create_remove_node_patch(old_element, new_element, path);
+        let remove_node_patches = old_element
+            .children
+            .iter()
+            .skip(new_child_count)
+            .enumerate()
+            .map(|(i, old_child)| {
+                Patch::remove_node(
+                    old_child.tag(),
+                    path.traverse(new_child_count + i),
+                )
+            })
+            .collect::<Vec<_>>();
+
         patches.extend(remove_node_patches);
     }
 
     patches
 }
 
-fn create_append_children_patch<'a, NS, TAG, LEAF, ATT, VAL>(
-    old_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
-    new_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
-    path: &[usize],
-) -> Patch<'a, NS, TAG, LEAF, ATT, VAL>
-where
-    NS: PartialEq + Clone + Debug,
-    LEAF: PartialEq + Clone + Debug,
-    TAG: PartialEq + Debug,
-    ATT: PartialEq + Clone + Debug,
-    VAL: PartialEq + Clone + Debug,
-{
-    let old_child_count = old_element.children.len();
-    let mut append_patch: Vec<&'a Node<NS, TAG, LEAF, ATT, VAL>> = vec![];
-
-    for append_child in new_element.children.iter().skip(old_child_count) {
-        append_patch.push(append_child);
-    }
-
-    Patch::append_children(
-        &old_element.tag,
-        TreePath::new(path.to_vec()),
-        append_patch,
-    )
-}
-
-fn create_remove_node_patch<'a, NS, TAG, LEAF, ATT, VAL>(
-    old_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
-    new_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
-    path: &[usize],
-) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
-where
-    NS: PartialEq + Clone + Debug,
-    LEAF: PartialEq + Clone + Debug,
-    TAG: PartialEq + Debug,
-    ATT: PartialEq + Clone + Debug,
-    VAL: PartialEq + Clone + Debug,
-{
-    let new_child_count = new_element.children.len();
-    let mut patches = vec![];
-    for (i, old_child) in old_element
-        .get_children()
-        .iter()
-        .skip(new_child_count)
-        .enumerate()
-    {
-        let mut child_path = path.to_vec();
-        child_path.push(new_child_count + i);
-        let remove_node_patch =
-            Patch::remove_node(old_child.tag(), TreePath::new(child_path));
-        patches.push(remove_node_patch);
-    }
-    patches
-}
-
 ///
 /// Note: The performance bottlenecks
 ///     - allocating new vec
@@ -440,7 +373,7 @@ where
 fn create_attribute_patches<'a, NS, TAG, LEAF, ATT, VAL>(
     old_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
     new_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
-    path: &[usize],
+    path: &TreePath,
 ) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
 where
     NS: PartialEq + Clone + Debug,
@@ -462,8 +395,6 @@ where
     // or the values differ
     // add it to the AddAttribute patches
     for (new_attr_name, new_attrs) in new_attributes_grouped.iter() {
-        // Issue: only the first found attribute's value is returned
-        // This could be problematic if there are multiple attributes of the same name
         let old_attr_values = old_attributes_grouped
             .iter()
             .find(|(att_name, _)| att_name == new_attr_name)
@@ -505,14 +436,14 @@ where
     if !add_attributes.is_empty() {
         patches.push(Patch::add_attributes(
             &old_element.tag,
-            TreePath::new(path.to_vec()),
+            path.clone(),
             add_attributes,
         ));
     }
     if !remove_attributes.is_empty() {
         patches.push(Patch::remove_attributes(
             &old_element.tag,
-            TreePath::new(path.to_vec()),
+            path.clone(),
             remove_attributes,
         ));
     }
