@@ -1,4 +1,4 @@
-use super::{create_attribute_patches, diff_recursive};
+use super::diff_recursive;
 use crate::{Element, Node, Patch, TreePath};
 use std::collections::BTreeMap;
 use std::fmt::Debug;
@@ -47,7 +47,7 @@ pub fn diff_keyed_elements<'a, 'b, NS, TAG, LEAF, ATT, VAL, SKIP, REP>(
     old_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
     new_element: &'a Element<NS, TAG, LEAF, ATT, VAL>,
     key: &ATT,
-    path: &[usize],
+    path: &TreePath,
     skip: &SKIP,
     rep: &REP,
 ) -> Vec<Patch<'a, NS, TAG, LEAF, ATT, VAL>>
@@ -90,10 +90,10 @@ where
 
     // check if there is no match from the keys in new_element to the keys in old_elements
     // if indeed there is no match at all, create a remove_all node and append all children
-    let has_match = new_key_index.iter().any(|(new_index, new_key)| {
+    let has_match = new_key_index.iter().any(|(_new_index, new_key)| {
         old_key_index
             .iter()
-            .any(|(old_index, old_key)| new_key == old_key)
+            .any(|(_old_index, old_key)| new_key == old_key)
     });
     // return early if there new no matches
     if !has_match {
@@ -102,11 +102,7 @@ where
             .iter()
             .enumerate()
             .map(|(old_index, old)| {
-                println!("There are no matches.. removing everything..");
-
-                let mut old_child_path = TreePath::new(path.to_vec());
-                old_child_path.push(old_index);
-                Patch::remove_node(old.tag(), old_child_path)
+                Patch::remove_node(old.tag(), path.traverse(old_index))
             })
             .collect::<Vec<_>>();
 
@@ -117,7 +113,7 @@ where
         if !for_append_children.is_empty() {
             let for_append_patch = Patch::append_children(
                 old_element.tag(),
-                TreePath::new(path.to_vec()),
+                path.clone(),
                 for_append_children,
             );
             patches.push(for_append_patch);
@@ -152,18 +148,10 @@ where
         // if there is a matching old_index, create a patch that will remove all the nodes
         // from the old_elements from the `last_matched_old_index` to this `matched_old_index
         if let Some(matched_old_index) = matched_old_index {
-            println!("--->> path: {:?}", path);
-            println!(
-                "matched old_index: {} to new_index: {}",
-                matched_old_index, new_index
-            );
-            let mut old_path = path.to_vec();
-            old_path.push(matched_old_index);
-
             let patch_for_matched = diff_recursive(
                 &old_element.children[matched_old_index],
                 new,
-                &old_path,
+                &path.traverse(matched_old_index),
                 key,
                 skip,
                 rep,
@@ -179,9 +167,7 @@ where
                     if is_forward(last_matched_old_index, i)
                         && i < matched_old_index
                     {
-                        let mut old_path = TreePath::new(path.to_vec());
-                        old_path.push(i);
-                        Some(Patch::remove_node(old.tag(), old_path))
+                        Some(Patch::remove_node(old.tag(), path.traverse(i)))
                     } else {
                         None
                     }
@@ -209,17 +195,13 @@ where
                 .collect::<Vec<_>>();
 
             if !for_insert_nodes.is_empty() {
-                let mut old_path = TreePath::new(path.to_vec());
-
                 let old_index_marked_node = last_matched_old_index.unwrap_or(0);
-                old_path.push(old_index_marked_node);
-
                 let old_tag = old_element.children[old_index_marked_node].tag();
                 // create a patch that will insert the new elements from `last_matched_new_index` to
                 // `new_index`
                 let for_insert_patch = Patch::insert_before_node(
                     old_tag,
-                    old_path,
+                    path.traverse(old_index_marked_node),
                     for_insert_nodes,
                 );
 
@@ -240,11 +222,7 @@ where
         .enumerate()
         .filter_map(|(i, old)| {
             if is_forward(last_matched_old_index, i) {
-                let mut old_path = TreePath::new(path.to_vec());
-                old_path.push(i);
-                println!("removing the remaining old node at: {:?}", old_path);
-
-                Some(Patch::remove_node(old.tag(), old_path))
+                Some(Patch::remove_node(old.tag(), path.traverse(i)))
             } else {
                 None
             }
@@ -255,13 +233,13 @@ where
         patches.extend(remaining_old_for_remove_patches);
     }
 
+    let old_index_marked_node = last_matched_old_index.unwrap_or(0);
+
     let old_tag = old_element
         .children
-        .get(last_matched_old_index.unwrap_or(0))
+        .get(old_index_marked_node)
         .map(|n| n.tag())
         .flatten();
-    let mut old_path = TreePath::new(path.to_vec());
-    old_path.push(last_matched_old_index.unwrap_or(0));
 
     // insert all the elements after the last_matched_new_index, insert it before the
     // node at last_matched_old_index
@@ -279,8 +257,11 @@ where
         .collect::<Vec<_>>();
 
     if !remaining_new_nodes.is_empty() {
-        let for_insert_after =
-            Patch::insert_after_node(old_tag, old_path, remaining_new_nodes);
+        let for_insert_after = Patch::insert_after_node(
+            old_tag,
+            path.traverse(old_index_marked_node),
+            remaining_new_nodes,
+        );
         patches.push(for_insert_after);
     }
 
@@ -288,6 +269,7 @@ where
 }
 
 // check if index is greater than the contained value of an index
+// returns true if `than` is None
 fn is_forward(than: Option<usize>, i: usize) -> bool {
     match than {
         None => true,
@@ -316,7 +298,7 @@ mod test {
             old.as_element_ref().unwrap(),
             new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -331,7 +313,7 @@ mod test {
             old.as_element_ref().unwrap(),
             new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::from([]),
             &|_, _| false,
             &|_, _| false,
         );
@@ -350,7 +332,7 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -370,7 +352,7 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -405,7 +387,7 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -437,14 +419,11 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
-        assert_eq!(
-            patches,
-            vec![Patch::remove_node(Some(&"div"), TreePath::new(vec![0]),)]
-        );
+        assert_eq!(patches, vec![Patch::remove_node(Some(&"div"), [0].into())]);
     }
 
     #[test]
@@ -465,13 +444,13 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
         assert_eq!(
             patches,
-            vec![Patch::remove_node(Some(&"div"), TreePath::new(vec![1]),)]
+            vec![Patch::remove_node(Some(&"div"), vec![1].into())]
         );
     }
 
@@ -501,7 +480,7 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -537,7 +516,7 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -586,7 +565,7 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -631,7 +610,7 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -683,7 +662,7 @@ mod test {
             &old.as_element_ref().unwrap(),
             &new.as_element_ref().unwrap(),
             &"key",
-            &[],
+            &TreePath::root(),
             &|_, _| false,
             &|_, _| false,
         );
@@ -693,16 +672,16 @@ mod test {
         assert_eq!(
             patches,
             vec![
-                Patch::remove_node(Some(&"div"), TreePath::new(vec![0])),
+                Patch::remove_node(Some(&"div"), TreePath::from([0])),
                 Patch::insert_before_node(
                     Some(&"div"),
-                    TreePath::new(vec![1]),
+                    TreePath::from([1]),
                     vec![&element("div", [attr("key", "10")], []),]
                 ),
-                Patch::remove_node(Some(&"div"), TreePath::new(vec![3])),
+                Patch::remove_node(Some(&"div"), TreePath::from([3])),
                 Patch::insert_after_node(
                     Some(&"div"),
-                    TreePath::new(vec![2]),
+                    TreePath::from([2]),
                     vec![&element("div", [attr("key", "40")], [])]
                 ),
             ]
