@@ -42,7 +42,7 @@ fn key_inserted_at_start() {
     assert_eq!(
         diff,
         vec![Patch::insert_node(
-            Some(&"main"),
+            Some(&"div"),
             TreePath::new(vec![0]),
             &element("div", vec![attr("key", "XXX")], vec![leaf("lineXXX")])
         )]
@@ -91,7 +91,7 @@ fn key_inserted_at_middle() {
     assert_eq!(
         diff,
         vec![Patch::insert_node(
-            Some(&"main"),
+            Some(&"div"),
             TreePath::new(vec![5]),
             &element("div", vec![attr("key", "XXX")], vec![leaf("lineXXX")])
         )]
@@ -147,7 +147,7 @@ fn wrapped_elements() {
     assert_eq!(
         diff,
         vec![Patch::insert_node(
-            Some(&"main"),
+            Some(&"div"),
             TreePath::new(vec![0, 5]),
             &element("div", vec![attr("key", "XXX")], vec![leaf("lineXXX")])
         )]
@@ -205,10 +205,10 @@ fn text_changed() {
     dbg!(&diff);
     assert_eq!(
         diff,
-        vec![Patch::replace_leaf(
+        vec![Patch::replace_node(
+            None,
             TreePath::new(vec![0, 6, 0]),
-            &"line7",
-            &"line7_changed"
+            &leaf("line7_changed")
         )]
     );
 }
@@ -261,10 +261,10 @@ fn text_changed_non_keyed() {
 
     assert_eq!(
         diff,
-        vec![Patch::replace_leaf(
+        vec![Patch::replace_node(
+            None,
             TreePath::new(vec![0, 6, 0]),
-            &"line7",
-            &"line7_changed"
+            &leaf("line7_changed")
         )]
     );
 }
@@ -355,11 +355,13 @@ fn insert_one_line_at_start() {
     assert_eq!(
         diff,
         vec![
-            Patch::replace_leaf(TreePath::new(vec![0, 0, 0, 0]), &"1", &"2"),
-            Patch::replace_leaf(TreePath::new(vec![0, 1, 0, 0]), &"2", &"3"),
-            Patch::replace_leaf(TreePath::new(vec![0, 2, 0, 0]), &"3", &"4"),
+            Patch::replace_node(
+                None,
+                TreePath::new(vec![0, 0, 0, 0]),
+                &leaf("2")
+            ),
             Patch::insert_node(
-                Some(&"main"),
+                Some(&"div"),
                 TreePath::new(vec![0, 0]),
                 &element(
                     "div",
@@ -369,7 +371,17 @@ fn insert_one_line_at_start() {
                         element("div", vec![], vec![leaf("XXX")]),
                     ],
                 ),
-            )
+            ),
+            Patch::replace_node(
+                None,
+                TreePath::new(vec![0, 1, 0, 0]),
+                &leaf("3")
+            ),
+            Patch::replace_node(
+                None,
+                TreePath::new(vec![0, 2, 0, 0]),
+                &leaf("4")
+            ),
         ]
     );
 }
@@ -469,32 +481,42 @@ fn insert_two_lines_at_start() {
     assert_eq!(
         diff,
         vec![
-            Patch::replace_leaf(TreePath::new(vec![0, 0, 0, 0]), &"1", &"3"),
-            Patch::replace_leaf(TreePath::new(vec![0, 1, 0, 0]), &"2", &"4"),
-            Patch::replace_leaf(TreePath::new(vec![0, 2, 0, 0]), &"2", &"5"),
-            Patch::insert_node(
-                Some(&"main"),
-                TreePath::new(vec![0, 0]),
-                &element(
-                    "div",
-                    vec![attr("key", "hashXXX")],
-                    vec![
-                        element("div", vec![], vec![leaf("1")]),
-                        element("div", vec![], vec![leaf("XXX")]),
-                    ],
-                ),
+            Patch::replace_node(
+                None,
+                TreePath::new(vec![0, 0, 0, 0]),
+                &leaf("3")
             ),
-            Patch::insert_node(
-                Some(&"main"),
+            Patch::insert_before_node(
+                Some(&"div"),
                 TreePath::new(vec![0, 0]),
-                &element(
-                    "div",
-                    vec![attr("key", "hashYYY")],
-                    vec![
-                        element("div", vec![], vec![leaf("2")]),
-                        element("div", vec![], vec![leaf("YYY")]),
-                    ],
-                )
+                vec![
+                    &element(
+                        "div",
+                        vec![attr("key", "hashXXX")],
+                        vec![
+                            element("div", vec![], vec![leaf("1")]),
+                            element("div", vec![], vec![leaf("XXX")]),
+                        ],
+                    ),
+                    &element(
+                        "div",
+                        vec![attr("key", "hashYYY")],
+                        vec![
+                            element("div", vec![], vec![leaf("2")]),
+                            element("div", vec![], vec![leaf("YYY")]),
+                        ],
+                    )
+                ]
+            ),
+            Patch::replace_node(
+                None,
+                TreePath::new(vec![0, 1, 0, 0]),
+                &leaf("4")
+            ),
+            Patch::replace_node(
+                None,
+                TreePath::new(vec![0, 2, 0, 0]),
+                &leaf("5")
             ),
         ]
     );
